/*
 * isolconsole.c
 *
 * Author:  Andy Cress  arcress@users.sourceforge.net
 * Copyright (c) 2006 Intel Corporation. 
 *
 * 10/12/03 Andy Cress - created, just a framework for now
 */
/*M*
Copyright (c) 2006, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

  a.. Redistributions of source code must retain the above copyright notice, 
      this list of conditions and the following disclaimer. 
  b.. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution. 
  c.. Neither the name of Intel Corporation nor the names of its contributors 
      may be used to endorse or promote products derived from this software 
      without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *M*/
#ifdef WIN32
#include <windows.h>
#include <stdio.h>
#include "getopt.h"
#else
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#endif
#include <string.h>
#include "ipmicmd.h"
 
/*
 * Global variables 
 */
char * progver   = "0.1";
char * progname  = "isolconsole";
char   fdebug    = 0;
char   fpicmg    = 0;
uchar ipmi_maj = 0;
uchar ipmi_min = 0;
uchar bSolVer  = 0;
uchar actcmd;
uchar netfn;


#ifdef WIN32
int __cdecl
#else
int
#endif
main(int argc, char **argv)
{
   int ret = 0;
   uchar devrec[16];
   uchar ibuf[64];
   uchar rbuf[64];
   int ilen, rlen;
   uchar cc;
   char c;
   int i;

   printf("%s ver %s\n", progname,progver);

   while ( (c = getopt( argc, argv,"adxET:V:N:P:R:U:?")) != EOF ) 
      switch(c) {
          case 'a': break;  /*activate*/
          case 'd': break;  /*deactivate*/
          case 'x': fdebug = 1;     break;  /* debug messages */
          case 'T':      /* auth type */
		i = atoi(optarg);  
                if (i > 0 && i <= 5) gauth_type = i;
                break;
          case 'V':      /* priv level */
		i = atoi(optarg);
                if (i > 0 && i <= 5) gpriv_level = i;
                break;
	  case 'N':      /* nodename */
		gnode = optarg;
		fipmi_lan = 1; 
		break;
	  case 'U':
		guser = strdup_(optarg);  /*remote username */
		if (!guser)  perror("strdup");
		else {  /* Hide username from 'ps' */
	        	memset(optarg, ' ', strlen(optarg));	
		}
		break;
 	  case 'R': 
 	  case 'P': 
		gpswd = strdup_(optarg);  /*remote password */
		if (!gpswd) perror("strdup");
		else {  /* Hide password from 'ps' */
	        	memset(optarg, ' ', strlen(optarg));	
		} 
 		break;
          case 'E':      /* get password from IPMI_PASSWORD environment var */
                gpswd = getenv("IPMI_PASSWORD");
                if (gpswd == NULL) perror("getenv(IPMI_PASSWORD)");
                else {
                   if (strlen(gpswd) > 16) gpswd[16] = 0;
                   if (fdebug) printf("using IPMI_PASSWORD\n");
                }
                break;
	  default:
                printf("Usage: %s [-adx -NUPRETV]\n", progname);
                printf(" where -a      activate SOL console session\n");
                printf("       -d      deactivate SOL console session\n");
                printf("       -x      show eXtra debug messages\n");
                printf("       -N node  Nodename of target system\n");
                printf("       -U user  Username for remote node\n");
                printf("       -P/-R pswd  Remote Password\n");
                printf("       -E    use password from Environment IPMI_PASSWORD\n");
                printf("       -T 1  use auth Type: 1=MD2, 2=MD5(default)\n");
                printf("       -V 2  use priVilege level: 2=user, 4=admin(default)\n");
                exit(1);
      }

   ret = ipmi_getdeviceid(devrec,16,fdebug);
   if (ret != 0) {
	printf("Cannot do ipmi_getdeviceid, ret = %d\n",ret);
	ipmi_close_();
	exit(1);
   } else {
      ipmi_maj = devrec[4] & 0x0f;
      ipmi_min = devrec[4] >> 4;
      printf("-- BMC version %x.%x, IPMI version %d.%d \n",
             devrec[2],  devrec[3], ipmi_maj, ipmi_min);
   }

   ret = ipmi_getpicmg(devrec,16,fdebug);
   if (ret == 0) fpicmg = 1;

   /* check for SOL support */
   if (ipmi_maj >= 2) bSolVer = 2;     /*IPMI 2.0*/
   else if (ipmi_maj == 1) {
      if (ipmi_min >= 5) bSolVer = 1;  /* IPMI 1.5 */
      else bSolVer = 0;  /* IPMI 1.0 */
   } else bSolVer = 0;
   if (bSolVer == 0) {
       printf("Serial Over Lan not supported for this IPMI version\n");
       exit(1);
   }
   /* May also want to verify that SOL is implemented here */

   /*
    * Spawn a console raw terminal thread now, which will wait for the 
    * "Activating cmd (0x02)" on success 
    */
   /* activate SOL = 0x01, get SOL status = 0x05 */
   if (bSolVer == 1) {
      actcmd  = (uchar)(ACTIVATE_SOL & 0x00ff); 
      netfn   = NETFN_SOL; 
      ibuf[0] = 0x01;   /* encryption | alerthandling | activateflag */
      ilen = 1;
   } else {   /* use IPMI 2.0 method */
      actcmd  = 0x01;
      netfn   = NETFN_TRANS;
      ibuf[0] = 0x01;   /* encryption | alerthandling | activateflag */
      ibuf[1] = 0;
      ibuf[2] = 0;
      ibuf[3] = 0;
      ibuf[4] = 0;
      ibuf[5] = 0;
      ilen = 6;
   }
   rlen = sizeof(rbuf);
   ret = ipmi_cmdraw(actcmd,netfn,BMC_SA, PUBLIC_BUS, BMC_LUN,
                   ibuf,ilen,rbuf,&rlen,&cc,fdebug);

   ipmi_close_();
   exit (ret);
}  /* end main()*/

/* end isolconsole.c */
